@@ -31,8 +31,9 @@ namespace Rcpp{
    - T has a SEXP constructor
 */
 template <typename T> T clone(const T& object) {
-    SEXP x = const_cast<T&>(object) ;
-    return T( Rf_duplicate( x ) ) ;
+    Shield<SEXP> x(const_cast<T&>(object));
+    Shield<SEXP> copy(Rf_duplicate(x));
+    return T((SEXP)copy);
 }
 
 } // namespace Rcpp
